#### Script for the Applied Session: "Networks Data Analysis: An Introduction #####
# by Andrea Deghi (University of Siena) email: andrea.deghi@unisi.it


#########  IGraph Package   #########

# install.packages(igraph)  # To install igraph package, remove '#'
library(igraph)

help(igraph)  # To get an overview of the functions of the package


#########   Loading and Creating Graphs   #########

# Loading from Pajek file (Notice: Networks, Partitions, Vectors etc can be sent directly from Pajek interface!)
getwd()
g <- read.graph(file="Example.net", format="pajek")  # Put the file in R directory first!

# Loading from Internet 
karate <- read.graph("http://cneurocvs.rmki.kfki.hu/igraph/karate.net", format="pajek")

# Basic Inspection

V(g) # Vertex Sequence
E(g) # Edge Sequence
summary(g) # Description of the Network

# Creating graphs

g <- graph.empty()       # Creates empty Graph
g <- graph.formula(A-B)  # Creates undirected Graph
g <- graph.formula(A-+B) # Directed Graph
g2 <- graph( c(1,2,2,3,3,4,5,6), directed=FALSE )

# Adding nodes and edges to an already existing graph, e.g.:
g <- graph.empty() + vertices(letters[1:10], color="red")
g <- g + vertices(letters[11:20], color="blue")
g <- g + edges(sample(V(g), 30, replace=TRUE), color="green")

# Graph Generators

g1 <- graph.star(10, mode="out")
g2 <- graph.lattice(c(5,5,5))
g3 <- graph.ring(10)
g4 <- graph.tree(10, 4)
g5 <- graph.full(5, loops=TRUE)
g6 <- graph.full.citation(10)
g7 <- graph.atlas(sample(0:1252, 1))

ws_graph <- watts.strogatz.game(1, 100, 4, 0.05)
ba_graph <- barabasi.game(100)

# Generate random graph with fixed probability
g <- erdos.renyi.game(100, 2/100)

# Generate random graph with fixed number of arcs
g <-erdos.renyi.game(20, 15, type='gnm')

# Generate preferential attachment graph
g <- barabasi.game(60, power=1, zero.appeal=1.3)

# Naming edges, ex. on a ring graph:

g <- graph.ring(10)   
V(g)$name <- sample(letters, vcount(g))


#########   Network Visualization   #########

# Standard Visualization 
g <- graph.tree(40, 4)
plot(g, layout=layout.circle)

# Force directed layouts
plot(g, layout=layout.fruchterman.reingold)
plot(g, layout=layout.graphopt)
plot(g, layout=layout.kamada.kawai)

# Interactive Visualization
tkplot(g, layout=layout.kamada.kawai)
l <- layout.kamada.kawai(g)

# 3D Visualization
rglplot(g, layout=l)

# Visual properties
plot(g, layout=l, vertex.color="cyan")


#########  Manipulation of Nodes and Edges Attributes Visualization  #########

g <- erdos.renyi.game(100, 1/100)
V(g)$color <- sample( c("red", "black"),vcount(g), rep=TRUE)
E(g)$color <- "grey"
red <- V(g)[ color == "red" ]
bl <- V(g)[ color == "black" ]
E(g)[ red ]$color <- "red"
E(g)[ bl ]$color <- "black"
plot(g, vertex.size=5, layout=layout.fruchterman.reingold)


#########  Measuring Graphs  #########

g <- erdos.renyi.game(100, 1/100)

# No of edges
length(E(g))

# Diameter
diameter(g)

# Density (No of edges / possible edges)
graph.density(g) 

# Number of islands
clusters(g)$no 

# Global cluster coefficient: (close triplets/all triplets)
transitivity(g, type = "global")

# Reciprocity of the graph
reciprocity(g)

#Average Path Length
average.path.length(g)

#Edge Connectivity
edge.connectivity(g)

# For transitivity
ws <- watts.strogatz.game(1, 100, 4, 0.05)
transitivity(ws)
p_hat <- ecount(ws)/(vcount(ws)*(vcount(ws))/2)
p_hat
er <- erdos.renyi.game(100, p_hat)
transitivity(er)


#########  Connected Component  #########

g <- erdos.renyi.game(100, 1/100)
subcomponent(g, 4, mode="out")  # Nodes reachable from node4
subcomponent(g, 4, mode="in")  # Nodes who can reach node4
clusters(g, mode="weak")
myc <- clusters(g, mode="strong")
mycolor <- c('green', 'yellow', 'red', 'skyblue')
V(g)$color <- mycolor[myc$membership + 1]
plot(g, layout=layout.fruchterman.reingold)

#########  Minimum Spanning Tree algorithm  #########  

# 1) Create the graph and assign random edge weights
g <-erdos.renyi.game(12, 0.35)
E(g)$weight <- round(runif(length(E(g))),2) * 50 
plot(g,layout=layout.fruchterman.reingold, edge.label=E(g)$weight)

# 2) Compute the minimum spanning tree
mst <- minimum.spanning.tree(g)
plot(mst, layout=layout.reingold.tilford, edge.label=E(mst)$weight)


#########  Centrality Measures  ######### 

# Degree Centrality
deg= degree(g)
deg_in = degree(g, mode = "in")
deg_out = degree(g, mode = "out")

# Closeness (inverse of average dist)
clo = closeness(g)

# Betweenness
bet = betweenness(g)

# Local cluster coefficient
tra = transitivity(g, type = "local")

# Eigenvector centrality
eig = evcent(g)$vector

net.cen = data.frame(degree = deg, closeness = clo, betweenness = bet, eigenvector = eig, 
                     transtivity = tra)
plot(net.cen)

# Plot the eigevector and betweenness centrality par(mfrow=c(1, 2))
plot(evcent(g)$vector, betweenness(g))
text(evcent(g)$vector, betweenness(g), cex = 0.6, pos = 4)


# Comparison between Centrality Measures: Eigenvector Centrality Vs Betweenes centrality
# 1)Create a graph
g <- barabasi.game(100, directed=F)
lay <- layout.fruchterman.reingold(g)

# Plot the eigevector and betweenness centrality
plot(evcent(g)$vector, betweenness(g))
text(evcent(g)$vector, betweenness(g), 0:100, cex=0.6, pos=4)
V(g)[12]$color <- 'red'
V(g)[8]$color <- 'green'
plot(g, layout=lay, vertex.size=8, vertex.label.cex=0.6)


# For degree distribution (Barabasi Network Vs Erdos and Renyi's Random Graph)

ba <- barabasi.game(10000, m=3)
p_hat <- ecount(ba)/ ((vcount(ba)-1)*vcount(ba)/2)
er <- erdos.renyi.game(10000, p_hat)
degree.distribution(er)
hist(degree(er))
hist(degree(ba))
plot(degree.distribution(er))
plot(degree.distribution(ba))


#######  Example of functions  ########

# You can create a function which gives you some basic statistics about networks. Here two examples
# Try to run on your network!

# Function for getting basic statistics

graph.basic_stats <- function(g) {
  # check g object
  if (!is.igraph(g)) stop("given grapg is not an igraph object")
  
  # compute some stats
  n <- vcount(g)                                  # number of nodes
  e <- ecount(g)  					 # number of edges 	
  deg <- degree(g)                                # degrees
  isol <- sum(deg==0)                             # number of isolated nodes
  comps <- clusters(g, mode="weak")               # Calculate the maximal (weakly or strongly) connected components of a graph
  
  # component sizes
  compsize <- sort(comps$csize, decreasing=TRUE)
  c1 <- compsize[1]  						# get the biggest component size
  c2 <- compsize[2]							# get the next biggest component size
  
  # print information
  cat("Number of nodes:", n, "\n")
  cat("Number of edges:", e, "\n\n")
  
  cat("Degree\n")
  cat("  Average:", mean(deg), "\n\n")
  
  cat("\n")
  cat("Giant component  Size:", c1, "\n")
  cat("Giant component  As perc.:", c1/n, "\n")
  cat("Second Giant component:", c2, "\n")
  cat("Second Giant component As perc.:", c2/n, "\n\n")
  
  cat("Isolated nodes\n")
  cat("  Number:", isol, "\n")
  cat("  As perc.:", isol/n, "\n\n")
}


#Plot and fit the power law distribution power law distribution

plot(degree.distribution(g), type = "b", xlab = "node degree", ylab = "Probability")
plot(degree.distribution(g), log = "xy", type = "b", xlab = "node degree", ylab = "probability")
# rank-order distribution
d = degree(graph, mode = "all")  ## Error: Not a graph object


fit_power_law = function(graph) {
  # calculate degree
  d = degree(graph, mode = "all")
  dd = degree.distribution(graph, mode = "all", cumulative = FALSE)
  degree = 1:max(d)
  probability = dd[-1]
  # delete blank values
  nonzero.position = which(probability != 0)
  probability = probability[nonzero.position]
  degree = degree[nonzero.position]
  reg = lm(log(probability) ~ log(degree))
  cozf = coef(reg)
  power.law.fit = function(x) exp(cozf[[1]] + cozf[[2]] * log(x))
  alpha = -cozf[[2]]
  R.square = summary(reg)$r.squared
  print(paste("Alpha =", round(alpha, 3)))
  print(paste("R square =", round(R.square, 3)))
  # plot
  plot(probability ~ degree, log = "xy", xlab = "Degree (log)", ylab = "Probability (log)", 
       col = 1, main = "Degree Distribution")
  curve(power.law.fit, col = "red", add = T, n = length(d))
  
}

# Try to run on your network simply by recalling the created function, with your net as an input!

graph.basic_stats(g)
fit_power_law(g)


